from django.conf import settings
from . import defaults


MARKYMARK_EXTENSIONS = getattr(
    settings,
    'MARKYMARK_EXTENSIONS',
    defaults.DEFAULT_MARKYMARK_EXTENSIONS
)

MARKYMARK_TEMPLATES = getattr(
    settings,
    'MARKYMARK_TEMPLATES',
    defaults.DEFAULT_MARKYMARK_TEMPLATES
)



import markdown
from django.utils.encoding import force_text
from django.utils.safestring import mark_safe
from django.utils import six

from markymark import conf
from markymark.extensions.base import MarkymarkExtension


class MarkymarkRenderer(markdown.Markdown):
    def registerExtensions(self, extensions, configs):
        """
        We are overwriting the default behavior of `markdown.Markdown`
        to provide some extensible plugin system and avoid failing
        because of stupid sublcass validation.

        `markdown` originally validates that instances that get passed to
        `registerExtension` are actual instances of `markdown.Extension`
        and if we're putting `MarkymarkExtension` in between this fails.
        """
        for ext in extensions:
            if isinstance(ext, six.string_types):
                ext = self.build_extension(ext, configs.get(ext, {}))
            if isinstance(ext, (markdown.Extension, MarkymarkExtension)):
                ext.extendMarkdown(self, globals())
            else:
                raise TypeError(
                    'Extension "%s.%s" must be of type: '
                    '"markdown.Extension" or "markymark.extensions.MarkymarkExtension"'
                    % (ext.__class__.__module__, ext.__class__.__name__))

        return self


def initialize_renderer(extensions=''):
    extensions = [e.strip() for e in extensions.split(',') if e]
    [extensions.append(module) for module in conf.MARKYMARK_EXTENSIONS]
    return MarkymarkRenderer(extensions=extensions)


def render_markdown(value, extensions=''):
    return mark_safe(initialize_renderer(extensions).convert(force_text(value)))



import re

import markdown
from django.template.loader import render_to_string

from markymark import conf
from markymark.extensions.base import MarkymarkExtension


AUTOLINK_RE = re.compile((
    r'(https?|ftps?|file|ssh|mms|svn(?:\+ssh)?|git|dict|nntp|irc|'
    r'rsync|smb|apt|mailto|telnet|s?news|sips?|skype|apt)'
    r'(://[-A-Za-z0-9+&@#/%?=~_()|!:,.;]*[-A-Za-z0-9+&@#/%=~_()|])'
))


class AutoLinkPostprocessor(markdown.postprocessors.Postprocessor):

    def run(self, text):
        def re_callback(match):
            return render_to_string(
                conf.MARKYMARK_TEMPLATES['autolink'],
                {'url': match.group()}
            )
        return AUTOLINK_RE.sub(re_callback, text)


class AutoLinkExtension(MarkymarkExtension):
    def extendMarkdown(self, md, md_globals):
        super(AutoLinkExtension, self).extendMarkdown(md, md_globals)
        md.postprocessors.add('autolink', AutoLinkPostprocessor(md), '_end')


def makeExtension(**kwargs):
    return AutoLinkExtension(**kwargs)



import factory

from filer.models.filemodels import File
from filer.models.imagemodels import Image


class FileFactory(factory.DjangoModelFactory):
    original_filename = 'file.data'
    file = factory.django.FileField(filename='file.data')

    class Meta:
        model = File


class ImageFactory(factory.DjangoModelFactory):
    original_filename = 'test.jpg'
    file = factory.django.ImageField(filename='test.jpg', color='blue')
    default_alt_text = 'test alt text'
    default_caption = 'test caption'

    class Meta:
        model = Image



from django import forms
from django.db import models

from .widgets import MarkdownTextarea


class MarkdownFormField(forms.fields.CharField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = MarkdownTextarea
        super(MarkdownFormField, self).__init__(*args, **kwargs)


class MarkdownField(models.TextField):
    def formfield(self, form_class=MarkdownFormField, **kwargs):
        return super(MarkdownField, self).formfield(
            form_class=form_class, **kwargs)


try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules(
        rules=[((MarkdownField,), [], {})],
        patterns=['^markymark\.fields']
    )
except ImportError:
    pass



from markymark.extensions.base import MarkymarkExtension


class CleanExtension(MarkymarkExtension):
    class Media:
        js = ('markdown/js/plugins/clean.js',)


def makeExtension(**kwargs):
    return CleanExtension(**kwargs)



from __future__ import absolute_import

from django import template
from django.utils.safestring import mark_safe

from markymark.utils import render_markdown


register = template.Library()


@register.filter(is_safe=True, name='markdown')
def markdown_filter(value, arg=''):
    return mark_safe(render_markdown(value, arg))



from django import forms

from markymark.utils import initialize_renderer


class MarkdownTextarea(forms.Textarea):
    def __init__(self, *args, **kwargs):
        super(MarkdownTextarea, self).__init__(*args, **kwargs)
        self.attrs['data-provide'] = 'markdown'

    def _media(self):
        media = forms.Media(
            css={'all': (
                'markdown/css/markdown-editor.css',
                'markdown/css/markdown-editor-adminfix.css',
            )},
            js=(
                'markdown/js/markdown.js',
                'markdown/js/markdown-editor.js'
            )
        )

        # Use official extension loading to initialize all extensions
        # and hook in extension-defined media files.
        renderer = initialize_renderer()

        for extension in renderer.registeredExtensions:
            if hasattr(extension, 'media'):
                media += extension.media
        return media

    media = property(_media)



from __future__ import absolute_import

import re

from django.conf import settings
from django.template.loader import render_to_string
from filer.models.filemodels import File
import markdown

from markymark import conf
from markymark.extensions.base import MarkymarkExtension


FILE_RE = re.compile(r'(\[file\:(?P<id>\d+)\])', re.IGNORECASE)


class FilerFileExtension(MarkymarkExtension):
    class Media:
        js = ('markdown/js/plugins/filer-file.js',)
        css = {
            'all': ('markdown/css/plugins/filer-file.css',)
        }

    def extendMarkdown(self, md, md_globals):
        super(FilerFileExtension, self).extendMarkdown(md, md_globals)
        md.postprocessors.add('filerfile', FilerFilePostprocessor(md), '_end')


class FilerFilePostprocessor(markdown.postprocessors.Postprocessor):
    """
    File markdown extension for django-filer for files and images.

    Usage:

      [file:id type:full pos:left|right]

    Position `pos` is optional.
    """

    def run(self, text):
        def re_callback(match):
            options = match.groupdict()
            try:
                file = File.objects.get(pk=int(options['id']))
                return render_to_string(conf.MARKYMARK_TEMPLATES['filer'], {
                    'file': file.get_real_instance(),
                })

            except File.DoesNotExist:
                if settings.DEBUG:
                    raise

            return match.group(0).replace(match.group(1), '')

        return FILE_RE.sub(re_callback, text)


def makeExtension(**kwargs):
    return FilerFileExtension(**kwargs)



import markdown
from django.forms.widgets import MediaDefiningClass
from django.utils import six


class MarkymarkExtension(six.with_metaclass(MediaDefiningClass), markdown.Extension):

    def extendMarkdown(self, md, md_globals):
        md.registerExtension(self)



import os
import sys
import codecs
from setuptools import setup, find_packages


version = '0.9.0'


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    print('You probably want to also tag the version now:')
    print('  git tag -a %s -m "version %s"' % (version, version))
    print('  git push --tags')
    sys.exit()


def read(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    with codecs.open(filename, encoding='utf-8') as fp:
        return fp.read()


install_requirements = [
    'django>=1.6',
    'markdown-py26-support==2.6.2'
]


test_requirements = [
    'pytest==2.8.0',
    'pytest-cache==1.0',
    'pytest-cov==2.1.0',
    'pytest-django==2.8.0',
    'pytest-flakes==1.0.1',
    'pytest-pep8==1.0.6',
    'cov-core==1.15.0',
    'mock==1.3.0',
    'factory-boy==2.5.2',
    'django-filer==0.9.12',
    'django-anylink==0.3.0',
]


setup(
    name='django-markymark',
    version=version,
    description=(
        'django-markymark provides helpers and tools to integrate markdown '
        'into your editor.'),
    long_description=read('README.rst'),
    author='Moccu GmbH & Co. KG',
    author_email='info@moccu.com',
    url='https://github.com/moccu/django-markymark/',
    packages=find_packages(exclude=[
        'markymark.tests',
        'examples',
        'examples.example',
        'examples.example.app',
    ]),
    install_requires=install_requirements,
    extras_require={
        'tests': test_requirements,
        'filer': ['django-filer==0.9.12', ],
        'anylink': ['django-anylink==0.3.0', ],
    },
    include_package_data=True,
    license='Apache License (2.0)',
    keywords=['markdown', 'django'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Framework :: Django',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    zip_safe=False,
)



DEFAULT_MARKYMARK_EXTENSIONS = [
    'markymark.extensions.autolink',
]


DEFAULT_MARKYMARK_TEMPLATES = {
    'filer': 'markdown/file.html',
    'anylink': 'markdown/anylink.html',
    'autolink': 'markdown/autolink.html',
}



from __future__ import absolute_import

import re

from anylink.models import AnyLink
from django.conf import settings
from django.template.loader import render_to_string
import markdown

from markymark import conf
from markymark.extensions.base import MarkymarkExtension


LINK_RE = re.compile(r'(\[link\:(?P<id>\d+)\])', re.IGNORECASE)


class AnyLinkExtension(MarkymarkExtension):
    class Media:
        js = (
            'markdown/js/plugins/anylink-link.js',
        )

    def extendMarkdown(self, md, md_globals):
        super(AnyLinkExtension, self).extendMarkdown(md, md_globals)
        md.postprocessors.add('anylink', AnyLinkPostprocessor(md), '_end')


class AnyLinkPostprocessor(markdown.postprocessors.Postprocessor):
    def run(self, text):
        def re_callback(match):

            options = match.groupdict()
            try:
                link = AnyLink.objects.get(pk=int(options['id']))
                return match.group(0).replace(
                    match.group(1),
                    render_to_string(conf.MARKYMARK_TEMPLATES['anylink'], {'link': link})
                )

            except (KeyError, AnyLink.DoesNotExist):
                if settings.DEBUG:
                    raise

        return LINK_RE.sub(re_callback, text)


def makeExtension(**kwargs):
    return AnyLinkExtension(**kwargs)
